"use strict";
const _path = /*#__PURE__*/ _interop_require_wildcard(require("path"));
const _javascriptobfuscator = require("javascript-obfuscator");
const _minimatch = require("minimatch");
const _webpack = /*#__PURE__*/ _interop_require_wildcard(require("webpack"));
const _type = require("./type");
function _define_property(obj, key, value) {
    if (key in obj) {
        Object.defineProperty(obj, key, {
            value: value,
            enumerable: true,
            configurable: true,
            writable: true
        });
    } else {
        obj[key] = value;
    }
    return obj;
}
function _getRequireWildcardCache(nodeInterop) {
    if (typeof WeakMap !== "function") return null;
    var cacheBabelInterop = new WeakMap();
    var cacheNodeInterop = new WeakMap();
    return (_getRequireWildcardCache = function(nodeInterop) {
        return nodeInterop ? cacheNodeInterop : cacheBabelInterop;
    })(nodeInterop);
}
function _interop_require_wildcard(obj, nodeInterop) {
    if (!nodeInterop && obj && obj.__esModule) {
        return obj;
    }
    if (obj === null || typeof obj !== "object" && typeof obj !== "function") {
        return {
            default: obj
        };
    }
    var cache = _getRequireWildcardCache(nodeInterop);
    if (cache && cache.has(obj)) {
        return cache.get(obj);
    }
    var newObj = {};
    var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor;
    for(var key in obj){
        if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) {
            var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null;
            if (desc && (desc.get || desc.set)) {
                Object.defineProperty(newObj, key, desc);
            } else {
                newObj[key] = obj[key];
            }
        }
    }
    newObj.default = obj;
    if (cache) {
        cache.set(obj, newObj);
    }
    return newObj;
}
const craeteError = (message = "An error occurred.")=>new Error(`${message} If you think this is a bug of nextjs-obfuscator, please file an issue at https://github.com/mtripg6666tdr/nextjs-obfuscator/issues`);
function main(obfuscatorOptions, pluginOptions) {
    // initialize options
    pluginOptions = Object.assign({
        enabled: "detect",
        patterns: [
            "./**/*.js",
            "./**/*.jsx",
            "./**/*.ts",
            "./**/*.tsx"
        ],
        log: false
    }, pluginOptions);
    pluginOptions.obfuscateFiles = Object.assign({
        buildManifest: false,
        ssgManifest: false,
        webpack: false,
        additionalModules: []
    }, pluginOptions.obfuscateFiles);
    const logger = pluginOptions.log ? console.log.bind(console, "\n") : ()=>{};
    const matchers = pluginOptions.patterns.map((pattern)=>new _minimatch.Minimatch(pattern));
    return function(nextConfig) {
        if (!nextConfig || typeof nextConfig !== "object") {
            throw new TypeError("Invalid configuration object passed.");
        }
        const moduleRegEx = /\.(tsx|ts|js|cjs|mjs|jsx)$/;
        const originalWebpackFn = nextConfig.webpack;
        nextConfig.webpack = function(config, context) {
            // @ts-ignore
            if (pluginOptions.writeConfig) {
                require("fs").writeFileSync(`./webpack-config-${Date.now()}.temp.json`, JSON.stringify(config, (key, value)=>value instanceof RegExp ? {
                        "[__type]": "RegExp",
                        value: value.toString()
                    } : typeof value === "bigint" ? {
                        "[__type": "BigInt",
                        value: value.toString()
                    } : value, "  "));
            }
            const basePath = config.context;
            if (!basePath) {
                throw craeteError("No context detected.");
            }
            const moduleMatcher = (value)=>{
                if (moduleRegEx.test(value)) {
                    if (!value.includes("node_modules")) {
                        const relativePath = `.${_path.sep}${_path.relative(basePath, value)}`;
                        logger("Detected:", relativePath);
                        if (matchers.some((matcher)=>{
                            const matched = matcher.match(relativePath);
                            if (matched) {
                                logger("Matched:", relativePath, `(pattern: ${matcher.pattern})`);
                            }
                            return matched;
                        })) {
                            return true;
                        }
                    } else {
                        const matched = pluginOptions.obfuscateFiles.additionalModules.some((mod)=>value.includes(`${_path.sep}node_modules${_path.sep}${mod}`));
                        if (matched) {
                            logger("Matched:", value);
                        }
                        return matched;
                    }
                }
                return false;
            };
            // obfuscate only if the chunk is for browser
            if (pluginOptions.enabled === true || pluginOptions.enabled === "detect" && !context.dev) {
                var _config_module;
                if (!context.isServer && !context.nextRuntime && ((_config_module = config.module) === null || _config_module === void 0 ? void 0 : _config_module.rules)) {
                    config.module.rules.unshift({
                        test: moduleMatcher,
                        enforce: "post",
                        use: {
                            loader: require.resolve("./loader"),
                            options: Object.assign({
                                [_type.LoggerSymbol]: logger
                            }, obfuscatorOptions)
                        }
                    });
                    const buildId = context.buildId;
                    if (!buildId) {
                        throw craeteError("No buildId found.");
                    }
                    if (Object.values(pluginOptions.obfuscateFiles).some((val)=>val)) {
                        config.plugins.push(new NextJSObfuscatorPlugin(obfuscatorOptions, buildId, pluginOptions.obfuscateFiles, logger));
                    }
                }
            }
            // by using below code we can obfuscator server components but at most cases it is not necessary
            // const appDirEnabled = (
            //   config.plugins?.find(p => p && typeof p === "object" && "appDirEnabled" in p) as ({ appDirEnabled: boolean } | undefined)
            // )?.appDirEnabled;
            // if(appDirEnabled && context.isServer && context.nextRuntime === "nodejs"){
            //   config.module!.rules!.unshift({
            //     test: moduleMatcher,
            //     enforce: "post",
            //     use: {
            //       loader: require.resolve("./loader"),
            //       options: Object.assign({
            //         [LoggerSymbol]: logger,
            //       }, obfuscatorOptions, {
            //         target: "node",
            //         domainLock: [],
            //       }),
            //     },
            //   });
            // }
            if (originalWebpackFn) {
                return originalWebpackFn(config, context);
            }
            return config;
        };
        return nextConfig;
    };
}
class NextJSObfuscatorPlugin {
    apply(compiler) {
        const PluginName = this.constructor.name;
        compiler.hooks.compilation.tap(PluginName, (compilation)=>{
            compilation.hooks.processAssets.tap({
                name: PluginName.slice(0, -6),
                stage: _webpack.default.Compilation.PROCESS_ASSETS_STAGE_DEV_TOOLING
            }, (assets)=>{
                this.logger("Initialized");
                const assetNames = Object.keys(assets);
                this.logger(assetNames);
                for(let i = 0; i < assetNames.length; i++){
                    const assetName = assetNames[i];
                    if (this.target.webpack && assetName.match(/^static\/chunks\/webpack-[abcdef\d]+\.js$/) || this.target.buildManifest && assetName === `static/${this.buildId}/_buildManifest.js` || this.target.ssgManifest && assetName === `static/${this.buildId}/_ssgManifest.js`) {
                        const asset = assets[assetName];
                        if (!asset) return;
                        const source = asset.source();
                        const obfuscatedCode = (0, _javascriptobfuscator.obfuscate)(source, this.obfuscatorOptions).getObfuscatedCode();
                        assets[assetName] = new _webpack.sources.RawSource(`!function(){${obfuscatedCode}}()`, false);
                        this.logger("Obfuscated:", assetName, source.length, "->", obfuscatedCode.length);
                    }
                }
            });
        });
    }
    constructor(obfuscatorOptions, buildId, target, logger){
        _define_property(this, "buildId", void 0);
        _define_property(this, "target", void 0);
        _define_property(this, "logger", void 0);
        _define_property(this, "obfuscatorOptions", void 0);
        this.buildId = buildId;
        this.target = target;
        this.logger = (...messages)=>logger("(Plugin)", ...messages);
        this.obfuscatorOptions = Object.assign(obfuscatorOptions, {
            splitStrings: false
        });
    }
}
module.exports = main;
